#include <stdio.h>
#include <stdlib.h>
#include <ft2build.h>
#include <freetype/freetype.h>
#include <freetype/ftsnames.h>
#include <freetype/ttnameid.h>
#include "SDL/SDL.h"
#include "SDL/SDL_ttf.h"
#include "fontlist.h"

#define WIDTH 300
#define HEIGHT 300

struct font_info {
	char *path;
	char *family;
};

#define FONT_NUM 16
struct font_info font_info_table[FONT_NUM];
int font_info_table_size;

/* 
 * Unicodeޥåפ ܸ(MS JAPANESE?) sfnt(?) եȤäƤեȤ 0 ʳ
 * ʤä 0 
 */

int 
check_font( FT_Face face )
{
	unsigned int i, charmap_num;
	FT_UInt snamec;

	if ( FT_IS_SFNT(face) == 0 )
		return 0;

	snamec = FT_Get_Sfnt_Name_Count(face);

	for ( i=0; i<snamec; i++ ) {
		FT_SfntName name;
		FT_Get_Sfnt_Name( face, i, &name );
		
		if ( name.language_id == TT_MS_LANGID_JAPANESE_JAPAN )
			break;
		
	}

	if ( i == snamec )
		return 0;

	charmap_num = face->num_charmaps;

	for ( i=0; i<charmap_num; i++ ) {
		FT_CharMap map = face->charmaps[i];

		if ( (map->encoding==FT_ENCODING_UNICODE) )
			return 1;
	}

	return 0;
}

void 
init_font_info_table()
{
	FontList *fl = fontlist_open();
	FT_Error err;
	FT_Library ftlib;

	int index=0;

	err = FT_Init_FreeType( &ftlib );

	if ( err != FT_Err_Ok ) {
		puts("ftlib error");
		abort();
	}

	while ( index<FONT_NUM ) {
		FT_Face face;
		char *path = fontlist_next_file( fl );

		if ( path == NULL )
			break;

		err = FT_New_Face( ftlib, path, 0, &face );
		if ( err != FT_Err_Ok )
			continue;
		
		if ( check_font(face) == 0)
			goto end;

		font_info_table[index].path = strdup(path);
		font_info_table[index].family = strdup(face->family_name);

		puts( face->family_name );
		puts( path );

		index++;

	end:
		FT_Done_Face( face );

	}

	font_info_table_size = index;

	FT_Done_FreeType( ftlib );
	fontlist_close( fl );
}

SDL_Surface *screen;
int font_index=0;
int font_size=12;

SDL_Color fgcolor = {    0,    0,    0 };
SDL_Color bgcolor = { 0xff, 0xff, 0xff };

void 
change_fontsize( int diff )
{
	static const int table[] = { 7, 8, 10, 12, 14, 16, 20, 28, 32 };
	static int cur=2;
	const int size = 9;
	
	cur += diff;

	if ( cur < 0 )
		cur = 0;
	else if ( cur >= size )
		cur = size-1;

	font_size = table[cur];
}

void 
change_font( int diff )
{
	font_index += diff;

	if ( font_index < 0 )
		font_index = 0;
	else if ( font_index >= font_info_table_size )
		font_index = font_info_table_size-1;
}

void 
update()
{
	unsigned int wid, ht;
	SDL_Surface *text_surface;
	SDL_Rect dist_rect;
	char *family = font_info_table[font_index].family;

	SDL_FillRect( screen, NULL, SDL_MapRGB(screen->format,0xff,0xff,0xff) );

	TTF_Font *font = TTF_OpenFont( font_info_table[font_index].path, font_size );
	TTF_SizeText( font, family, &wid, &ht );
	
	text_surface = TTF_RenderText( font, family, fgcolor, bgcolor );

	dist_rect.w = wid;
	dist_rect.h = ht;
	dist_rect.x = (WIDTH/2)-(wid/2);
	dist_rect.y = (HEIGHT/2)-(ht/2);

	SDL_BlitSurface( text_surface, NULL, screen, &dist_rect );
	
	SDL_FreeSurface( text_surface );
	TTF_CloseFont( font );

	SDL_UpdateRect( screen, 0, 0, WIDTH, HEIGHT );
}	

void
keydown( SDL_KeyboardEvent *ev )
{
	switch( ev->keysym.sym ) {
	case SDLK_ESCAPE:
		exit(0);

	case SDLK_UP:
		change_fontsize( 1 );
		update();
		break;

	case SDLK_DOWN:
		change_fontsize( -1 );
		update();
		break;

	case SDLK_RIGHT:
		change_font( 1 );
		update();
		break;

	case SDLK_LEFT:
		change_font( -1 );
		update();
		break;

	default:
		break;
	}
}

int 
main()
{
	SDL_Init( SDL_INIT_VIDEO );
	TTF_Init();

	screen = SDL_SetVideoMode( WIDTH, HEIGHT, 0, SDL_SWSURFACE|SDL_DOUBLEBUF );
	
	init_font_info_table( );

	atexit( SDL_Quit );

	update();

	while ( 1 ) { 
		SDL_Event ev;

		SDL_WaitEvent( &ev );

		switch ( ev.type ) {
		case SDL_KEYDOWN:
			keydown( (SDL_KeyboardEvent*)&ev );
			break;
		default:
			break;
		}

	}

	return 0;
}
